#!/bin/ksh93
#
###############################################################################
# SHELL SCRIPT SOURCE FILE
#
# PROGRAM NAME :       HealthCheck.sh
# DESCRIPTION :        This script has been developed to monitor and report the
#                      UNIX components. This script builds on an AIX script
#
###############################################################################
#
###############################################################################
# CHANGE DESCRIPTION :    Draft version with FIX option
# 27/09/2007 - Sufyaan Kazi (CA Wily) - Tidied up script
#
#############################################################################################

# Global variables

HOSTNAME=`hostname`
DISKIGNOREFILE=/usr/local/mware/system/$HOSTNAME/diskignores.txt
DATE=`date +%d-%m-%Y:%H:%m`

#Function to write a metric in XML format for the EPA
# Arg 1 - Metric Name and Value (e.g. User=40)
# Arg 2 - Sub-Folder to stitch the metric in the Investigator Tree under HealthCheck
# Arg 3 - Metric Type, e.g. LongCounter
write_metric()
{
        print "<metric type=\"${1}\"  name=\"AIX|${2}:${3}\" value=\"${4}\"/>"
}

#Function to get the total Paging Space and %Used
PAGING_status()
{
        #PAGING_TOT=`lsps -s | tail -1 | cut -d" " -f7| cut -d"MB" -f1`
        #PAGING_USED=`lsps -s | tail -1 | cut -d" " -f21| cut -d"%" -f1`

        lsps -s | tail +2 | read pf pu
        pf=${pf%MB}
        pu=${pu%\%}

        write_metric IntCounter Settings pageFileSize $pf
        write_metric IntCounter Memory pageFilePercUsed $pu

        vmstat -v | awk -f /usr/local/mware/iscope/scripts/AIX/vmstat.awk 
}

#Function to display machine information
MACHINEINFO()
{
      uname -a | while read os hostname osmin osmaj machine rest
      do
          write_metric StringEvent Settings os $os
          write_metric StringEvent Settings hostname $hostname
          write_metric StringEvent Settings osmin $osmin
          write_metric StringEvent Settings osmaj $osmaj
          write_metric StringEvent Settings machine $machine
      done
}


#Function to count the no. of processes and zombies
NUM_Zombies()
{
        #Of which, no. of zombies
        NUM_ZOMBIES=`ps -efa | awk '{ if ( $1 != "root" && $2 == "1" && $3 == "1" ){ print $0 } } /defunct/ { print $0 } ' | wc -l`
        NUM_ZOMBIES=`expr $NUM_ZOMBIES - 1`

        write_metric IntCounter Kernel procZombies $NUM_ZOMBIES
}

#Function to view Filesystem used values
FILESYS_status()
{
        #for FILESYS in `mount | egrep -v "nfs|procfs|cdrfs|mounted|\-\-" | awk '{ if (NR>1){ print $2 }}'`
        #   diskspc= `df -k ${FILESYS} | grep -iv mount | awk '{ printf ("%s%s=%s\n","FILESYS_",$7,$4) }' 2>/dev/null | cut -d"%" -f1` $FS

        savedf=/tmp/df${USER}.tmp

        #
        #  wait for df command which can hang with NFS mounts
        #
        df | egrep -v '\:|^/proc|Mounted' > $savedf &
        lp=$!
        
        cnt=1
        dfok=1
        while [[ $cnt -lt 10 ]]
        do
          ps -p $lp > /dev/null 2>&1
          if [ $? -gt 0 ];then
            dfok=0
            cnt=100
          else
            sleep 1
            cnt=$(($cnt + 1))
          fi
        done
       
        #
        #  Kill df if it doesnt return in 3 seconds
        #
        if [ $dfok == 1 ];then
        
          kill $lp
          write_metric IntCounter DiskSpace dfhang 1

        else
         
          write_metric IntCounter DiskSpace dfhang 0

          checkignores=n
          if [ -e $DISKIGNOREFILE ];then
             checkignores=y
          fi

          cat $savedf | while read mnt size free pct iused ipct dir
          do
             pct=${pct%\%}
             ipct=${ipct%\%}
             base=DiskSpace
             ignoremnt=n

             if [ "$checkignores" == "y" ];then
               cat $DISKIGNOREFILE | while read mmnt ignore
               do
                  xx=$(( ${#mmnt} ))
                  mdir=${dir:0:$xx}

                  if [ "$mdir" == "$mmnt" ]; then
                     base=DiskSpaceX
                     if [ "$ignore" == "y" ]; then
                        ignoremnt=y
                     fi
                  fi
               done
             fi

             if [ "$ignoremnt" == "n" ]; then

                write_metric IntCounter $base\|$dir size $size
                write_metric IntCounter $base\|$dir percUsed $pct
                write_metric IntCounter $base\|$dir percInodesUsed $ipct

                #write_metric IntCounter DISK $dir\:size $iused
                #write_metric IntCounter DISK $dir\:size $free

             fi
          done


        fi
        
        rm $savedf
}

#Function to view Error Reports for the last n hours
ERROR_status()
{
# TIME is being set here to look over the last hour, this period can be changed by simply changing the following
        #TIME=`TZ=GMT+1 date +%m%d%H%M%y`
        min=`date +%M`
        min=$(($min-1))
        TIME=`date +%m%d%H$min%y`
        NO_ERRS=`errpt -s $TIME | grep -v IDENTIFIER | wc -l `
        write_metric PerIntervalCounter Availability fatalErrors $NO_ERRS
}

# Main script

. /usr/local/mware/bin/ksh.function_library

dontRunMeTwice
trap exitTrap 1 2 3 6 9 14 15

MACHINEINFO
NUM_Zombies
FILESYS_status
ERROR_status
PAGING_status
#VMSTAT

dontRunMeTwice rm

